module ConfigFile where

import Char
import System
import Concurrent
import IOExts(unsafePerformIO)

type ConfigFile = [(String, String)]

--configLookup :: ConfigFile -> String -> String
--configLookup cf s = case lookup s cf of
--    Just v -> v
--    Nothing -> ""

getConfiguration :: String -> IO ConfigFile
getConfiguration s = do
    e <- getEnv "HOME"
    c <- readFile (e ++ "/" ++ s)
    return $ parseConfigFile c
    
dumpConfigFile :: ConfigFile -> String
dumpConfigFile cf = unlines $ map dc cf where
    dc (n,v) = n ++ " " ++ v

mergeConfigFile :: ConfigFile -> ConfigFile -> ConfigFile
mergeConfigFile c1 c2 = c1 ++ c2

isPropName c = isAlphaNum c || c `elem` "-_"

parseConfigFile :: String -> ConfigFile
parseConfigFile s = concatMap bl (lines (uncomment s)) where
    uncomment ('-':'-':xs) = uncomment (dropWhile (/= '\n') xs)
    uncomment ('/':'*':xs) = uncomment (dropEnd xs)  
    uncomment (x:xs) = x:uncomment xs
    uncomment [] = []
    dropEnd ('*':'/':xs) = uncomment xs
    dropEnd (_:xs) = dropEnd xs
    dropEnd [] = []
    bl s = let (n,r) = span isPropName (dropWhile isSpace s) in
	if null n then [] else [(n,dropWhile isSpace r)]


{-# NOINLINE config_file_var #-}
config_file_var :: MVar ConfigFile
config_file_var = unsafePerformIO $ newMVar []

{-# NOINLINE config_file #-}
config_file = unsafePerformIO $ readMVar config_file_var

loadStdConfiguration :: String -> IO ()
loadStdConfiguration f = do
    c <- getConfiguration f
    swapMVar config_file_var c
    return ()

loadExactConfiguration :: String -> IO ()
loadExactConfiguration f = do
    s <- readFile f 
    let c = parseConfigFile s
    swapMVar config_file_var c
    return ()


stdLookup v = lookup v config_file
stdLookupList v = maybe [] words $ stdLookup v 

configLookup k = do
    vs <- readMVar config_file_var
    return $ lookup k vs


